# joaddons.tcl for netbots.tcl v3.15
# designed to work with eggdrop 1.5.2 or higher
# Johoho's Eggdrop Page - http://johoho.eggdrop.cx
# joaddons.tcl copyright (c) 1999,2000 by johoho <johoho@hojo-net.de>
# netbots.tcl copyright (c) 1998-2000 by slennox <slennox@egghelp.org>

## joaddons.tcl component script v2.00.1, 19.03.2000 ##

# History:
# 
# v1.00.0, 27.12.1999 - initial release, not public
# v2.00.0, 03.02.2000 - added lots of stuff and put it on homepage
# v2.00.1, 19.03.2000 - change header and readme :)

# Todo:
# 
# - add a check for /part msg colours
# - add caps support
# - add a command for changing color status manually (?)
# - unify code, so I don't need to write extra proc for each color task

# thanks to BASS, G'Quann, NML_375 and toot for their help. thanks slennox for netbots.tcl


proc ja_helpidx {hand chan idx} {
global ja_local ja_remote
  if {![matchattr $hand o|o $chan]} {return 0}
  putidx $idx "joaddons.tcl commands"
  if {$ja_local} {
   putidx $idx "local commands"
   if {[matchattr $hand n]} {
    putidx $idx " For owners:"
    putidx $idx "  copyflags     mchattr"
   }
   if {[matchattr $hand m]} {
    putidx $idx " For masters:"
    putidx $idx "  checkhosts    checkpass"
   }
  putidx $idx " "
  }
  if {$ja_remote} {
   putidx $idx "remote commands"
   if {[matchattr $hand n]} {
    putidx $idx " For owners:"
    putidx $idx "  r-user        r+user        r-host        r+host"
    putidx $idx "  r-bot         r+bot         rchattr       rbotattr"
    putidx $idx "  rrestart      rdie"
   }
  }
  putidx $idx " "
  return 1
}

proc ja_help {hand chan idx cmd} {
  if {[matchattr $hand o|o $chan]} {
    switch -exact -- $cmd {
      "checkhosts" {
        if {[matchattr $hand m]} {
          putidx $idx "# checkhosts <count>"
          putwrap $idx 3 "This command lists all users with or with more then <count> hosts."
          return 1
        }
      }
      "checkpass" {
        if {[matchattr $hand m]} {
          putidx $idx "# checkpass"
          putwrap $idx 3 "This command lists all users who haven't got a password set."
          return 1
        }
      }
      "copyflags" {
        if {[matchattr $hand n]} {
          putidx $idx "# copyflags <#fromchan> <#tochan>"
          putwrap $idx 3 "This command lets you copy all user flags from <#fromchan> to <#tochan>"
          return 1
        }
      }
      "mchattr" {
        if {[matchattr $hand n]} {
          putidx $idx "# mchattr <currentflags> <newflags> \[#channel\]"
          putwrap $idx 3 "This commands lets you chattr people by matching flags. If you specify a channel, it will copy only channel flags."
          return 1
        }
      }
      "r+bot" {
        if {[matchattr $hand n]} {
          putidx $idx "# r+bot <botname> \[adress:port\]"
          putwrap $idx 3 "This command adds a bot to userlist with optional adress and telnet port specified."
          return 1
        }
      }
      "r-bot" {
        if {[matchattr $hand n]} {
          putidx $idx "# r-bot <bot>"
          putwrap $idx 3 "This command removes a bot user entry."
          return 1
        }
      }
      "r-host" {
        if {[matchattr $hand n]} {
          putidx $idx "# r-host <nickname> <hostmask>"
          putwrap $idx 3 "This command removes the hostmask from nickname."
          return 1
        }
      }
      "r-user" {
        if {[matchattr $hand n]} {
          putidx $idx "# r-user <user>"
          putwrap $idx 3 "This command deletes an user entry."
          return 1
        }
      }
      "r+bot" {
        if {[matchattr $hand n]} {
          putidx $idx "# r+bot <bot> \[adress:port\]"
          putwrap $idx 3 "This command adds a bot entry to userfile."
          return 1
        }
      }
      "r+host" {
        if {[matchattr $hand n]} {
          putidx $idx "# r+host <nickname> <hostmask>"
          putwrap $idx 3 "This command adds a hostmask to nickname."
          return 1
        }
      }
      "r+user" {
        if {[matchattr $hand n]} {
          putidx $idx "# r+user <user> <hostmask>"
          putwrap $idx 3 "This command adds an user entry."
          return 1
        }
      }
      "rbotattr" {
        if {[matchattr $hand n]} {
          putidx $idx "# rbotattr <bot> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
          putwrap $idx 3 "This command allows the remote setting of channel or global flags for a bot. The channel setting is optional."
          return 1
        }
      }
      "rchattr" {
        if {[matchattr $hand n]} {
          putidx $idx "# rchattr <nickname> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
          putwrap $idx 3 "This command allows the remote setting of channel or global flags for a user. The channel setting is optional."
          return 1
        }
      }
      "rdie" {
        if {[matchattr $hand n]} {
          putidx $idx "# rdie really \[reason\]"
          putwrap $idx 3 "This kills all bots. You must put a 'really' after rdie, because it's a very dangerous command. You can optional supply a quitmsg."
          return 1
        }
      }
      "rrestart" {
        if {[matchattr $hand n]} {
          putidx $idx "# rrestart"
          putwrap $idx 3 "This command restarts each bot."
          return 1
        }
      }
    }
  }
  return 0
}

lappend nb_helpidx "ja_helpidx"
set nb_help(checkhosts) "ja_help"
set nb_help(checkpass) "ja_help"
set nb_help(copyflags) "ja_help"
set nb_help(mchattr) "ja_help"
set nb_help(r-bot) "ja_help"
set nb_help(r-host) "ja_help"
set nb_help(r-user) "ja_help"
set nb_help(r+bot) "ja_help"
set nb_help(r+host) "ja_help"
set nb_help(r+user) "ja_help"
set nb_help(rbotattr) "ja_help"
set nb_help(rchattr) "ja_help"
set nb_help(rdie) "ja_help"
set nb_help(rrestart) "ja_help"



# checkhosts, v1.0
# shows users with more than x hosts
proc ja_checkhosts {handle idx arg} {
   set hosts $arg
   if {$hosts == ""} {
      putdcc $idx "usage: .checkhosts <hosts>"
      return
   }
   append list ""
   putcmdlog "#$handle# hostcheck $hosts"
   set usercount 0
   foreach user [userlist] {
      set hostcount 0
      foreach host [getuser $user hosts] {
         incr hostcount 1
      }
      if {$hosts == 0} {
        if {$hostcount == $hosts} {
          append list "$user, "
          incr usercount 1
        }      
      } else {
        if {$hostcount >= $hosts} {
          append list "$user, "
          incr usercount 1
        }
      }
   }
   if {$list >= 1} {
      if {$hosts == 0} {
        putdcc $idx "$usercount users with $hosts hosts: [string trimright $list ", "]"
      } else {
        putdcc $idx "$usercount users with $hosts or more hosts: [string trimright $list ", "]"
      }
   } else {
      if {$hosts == 0} {
        putdcc $idx "no users found with $hosts hosts"
      } else {
        putdcc $idx "no users found with $hosts or more hosts"
      }
   }
}


# checkpass, v1.0
# checks for users without password
proc ja_checkpass {hand idx arg} {
  set count 0
  set useruser ""
  putcmdlog "#$hand# checkpass"
  foreach useruser [userlist] {
    if {[passwdok $useruser ""]} {
      append list "$useruser, "
      incr count 1
    }
  }
  if {$count >= 1} {
    if {$count == 1} {
      putdcc $idx "There is $count user without a password: $useruser"
    } else {
      putdcc $idx "There are $count users without a password: [string trimright $list ", "]"
    }
  } else {
    putdcc $idx "There are no users without a password set."
  }
  return 1
}


# copyflag v1.0
# This command lets you copy all user flags from one channel to another

proc ja_copyflags {hand idx arg} {
   set ja_fromchan [lindex [split $arg] 0]
   set ja_tochan [lindex [split $arg] 1]
   if {$ja_fromchan == "" || $ja_tochan == ""} {
      putdcc $idx "usage: .copyflags #fromchan #tochan"
      return 0
   }
   if {![validchan $ja_fromchan]} {
      putdcc $idx "$ja_fromchan is not a valid channel"
      return 0
   }
   if {![validchan $ja_tochan]} {
      putdcc $idx "$ja_tochan is not a valid channel"
      return 0
   }
   foreach user [userlist] {
      set flags [chattr $user $ja_fromchan]
      chattr $user $flags $ja_tochan
   }
   putcmdlog "#$hand# copyflags $ja_fromchan $ja_tochan"
   putdcc $idx "flags sucessfully copied from $ja_fromchan to $ja_tochan"
   return 1
}


# mchattr v1.0
# This command lets you copy all user flags from one channel to another

proc ja_mchattr {hand idx arg} {
   set ja_mcoflags [lindex [split $arg] 0]
   set ja_mcnflags [lindex [split $arg] 1]
   set ja_mcchan [lindex [split $arg] 2]
   if {$ja_mcoflags == "" || $ja_mcnflags == ""} {
      putdcc $idx "usage: .mchattr <oldflags> <newflags> \[channel\]"
      return 0
   }
   if {![validchan $ja_mcchan] && $ja_mcchan != ""} {
      putdcc $idx "$ja_mcchan is not a valid channel"
      return 0
   }
   putcmdlog "#$hand# mchattr $arg"
   set changed 0
   if {$ja_mcchan != ""} {
      foreach user [userlist |$ja_mcoflags $ja_mcchan] {
         chattr $user |$ja_mcnflags $ja_mcchan
         incr changed 1
      }
      if {$changed >= 1} {
         putdcc $idx "changed $changed users with $ja_mcoflags to $ja_mcnflags on $ja_mcchan"
      } else {
         putdcc $idx "no users with $ja_mcoflags $ja_mcchan found"
      }
   } else {
      foreach user [userlist $ja_mcoflags] {
          chattr $user $ja_mcnflags
          incr changed 1
      }
      if {$changed >= 1} {
         putdcc $idx "changed $changed users with $ja_mcoflags to $ja_mcnflags"
      } else {
         putdcc $idx "no users with $ja_mcoflags found"
      }
   }
}


# remote, v1.1
# remote command availability

# r-bot v1.0
proc ja_dccr-bot {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
    if {($victim == "")} {
      putdcc $idx "Usage: .-bot <bot>"
      return 0    
    }
    putcmdlog "#$hand# r-bot $arg"
    nb_sendcmd $nb_control ja_netr-bot "$hand $idx $victim"
    if {[nb_control]} {
      deluser $victim
    }
}

proc ja_netr-bot {frombot arg} {
    set uidx [slindex $arg 1]
    set victim [slindex $arg 2]
    if {[deluser $victim]} {
      putlog "netbots \[[slindex $arg 0]@$frombot\]: -bot $victim"
      nb_sendcmd $frombot ja_netr-botz "$hand $uidx 0 $victim"
    } else {
      nb_sendcmd $frombot ja_netr-botz "$hand $uidx 1 $victim"
    }
}

proc ja_netr-botz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    if {($result == 0)} {
      putdcc $uidx "r-bot successfull - deleted $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "r-bot failed - $victim is not a valid bot"
      return 0
    }
}

# r-host v1.0
proc ja_dccr-host {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set hmask [lindex $arg 1]
    if {($victim == "") || ($hmask == "")} {
      putdcc $idx "Usage: .r-host <nickname> <hostmask>"
      return 0    
    }
    putcmdlog "#$hand# r-host $arg"
    nb_sendcmd $nb_control ja_netr-host "$hand $victim $hmask $idx"
    if {[nb_control]} {
      delhost $victim $hmask
    }
}

proc ja_netr-host {frombot arg} {
global nb_control
    set hand [slindex $arg 0]
    set victim [slindex $arg 1]
    set hmask [slindex $arg 2]
    set uidx [slindex $arg 3]
    if {[validuser $victim]} {
	  if {[delhost $victim $hmask]} {
        putlog "netbots \[[slindex $arg 0]@$frombot\]: -host $victim $hmask $uidx"
        nb_sendcmd $frombot ja_netr-hostz "$hand $uidx 0 $victim $hmask"
      } else {
        nb_sendcmd $frombot ja_netr-hostz "$hand $uidx 2 $victim $hmask"
      }
    } else {
      nb_sendcmd $frombot ja_netr-hostz "$hand $uidx 1 $victim $hmask"
    }
}

proc ja_netr-hostz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    set hmask [slindex $arg 4]
    if {($result == 0)} {
      putdcc $uidx "r-host successfull - removed $hmask from $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "r-host failed - $victim is not a valid nickname"
      return 0
    } elseif {($result == 2)} {
      putdcc $uidx "r-host failed - $hmask is not a valid hostmask for $victim"
      return 0
    }
}

# r-user v1.0
proc ja_dccr-user {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
    if {($victim == "")} {
      putdcc $idx "Usage: .-user <user>"
      return 0    
    }
    putcmdlog "#$hand# r-user $arg"
    nb_sendcmd $nb_control ja_netr-user "$hand $idx $victim"
    if {[nb_control]} {
      deluser $victim
    }
}

proc ja_netr-user {frombot arg} {
    set uidx [slindex $arg 1]
    set victim [slindex $arg 2]
    if {[deluser $victim]} {
      putlog "netbots \[[slindex $arg 0]@$frombot\]: -user $victim"
      nb_sendcmd $frombot ja_netr-userz "$hand $uidx 0 $victim"
    } else {
      nb_sendcmd $frombot ja_netr-userz "$hand $uidx 1 $victim"
    }
}

proc ja_netr-userz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    if {($result == 0)} {
      putdcc $uidx "r-user successfull - deleted $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "r-user failed - $victim is not a valid user"
      return 0
    }
}

# r+bot v1.0
proc ja_dccr+bot {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set adrip [lindex $arg 1]
    if {($victim == "")} {
      putdcc $idx "Usage: .+user <user> <adress:ip>"
      return 0    
    }
    putcmdlog "#$hand# r+bot $victim $adrip"
    nb_sendcmd $nb_control ja_netr+bot "$hand $idx $victim $adrip"
    if {[nb_control]} {
      if {($channel == "")} {
        adduser $victim
      } else {
        adduser $victim $adrip
      }
    }
}

proc ja_netr+bot {frombot arg} {
    set uidx [slindex $arg 1]
    set victim [slindex $arg 2]
    set adrip [slindex $arg 3]
    if {[validuser $victim]} {
      nb_sendcmd $frombot ja_netr+botz "$hand $uidx 1 $victim"
    } else {
      if {($adrip == "")} {
        if {[addbot $victim]} {
          putlog "netbots \[[slindex $arg 0]@$frombot\]: +user $victim"
          nb_sendcmd $frombot ja_netr+botz "$hand $uidx 0 $victim"
        } else {
          nb_sendcmd $frombot ja_netr+botz "$hand $uidx 2 $victim"
        }
      } else {
        if {[addbot $victim $adrip]} {
          putlog "netbots \[[slindex $arg 0]@$frombot\]: +user $victim $adrip"
          nb_sendcmd $frombot ja_netr+botz "$hand $uidx 0 $victim $adrip"
        } else {
          nb_sendcmd $frombot ja_netr+botz "$hand $uidx 2 $victim $adrip"
        }
     }
  }
}

proc ja_netr+botz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    set adrip [slindex $arg 3]
    if {($result == 0)} {
      if {($adrip == "")} {
        putdcc $uidx "r+bot successfull - added $victim"
        return 1
      } else {
        putdcc $uidx "r+bot successfull - added $victim $adrip"
        return 1
      }
    } elseif {($result == 1)} {
      putdcc $uidx "r+bot failed - $victim is already in userfile"
      return 0
    } elseif {($result == 2)} {
      if {($adrip == "")} {
        putdcc $uidx "r+bot failed - Couldn't add $victim to userfile"
        return 0
      } else {
        putdcc $uidx "r+bot failed - Couldn't add $victim with $adrip to userfile"
        return 0       
      }
    }
}

# r+host v1.0
proc ja_dccr+host {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set hmask [lindex $arg 1]
    if {($victim == "") || ($hmask == "")} {
      putdcc $idx "Usage: .r+host <nickname> <hostmask>"
      return 0    
    }
    putcmdlog "#$hand# r+host $arg"
    nb_sendcmd $nb_control ja_netr+host "$hand $victim $hmask $idx"
    if {[nb_control]} {
      setuser $victim HOSTS $hmask
    }
}

proc ja_netr+host {frombot arg} {
global nb_control
    set hand [slindex $arg 0]
    set victim [slindex $arg 1]
    set hmask [slindex $arg 2]
    set uidx [slindex $arg 3]
    if {[validuser $victim]} {
	  if {[setuser $victim HOSTS $hmask]} {
        putlog "netbots \[[slindex $arg 0]@$frombot\]: +host $victim $hmask $uidx"
        nb_sendcmd $frombot ja_netr+hostz "$hand $uidx 0 $victim $hmask"
      } else {
        nb_sendcmd $frombot ja_netr+hostz "$hand $uidx 2 $victim $hmask"
      }
    } else {
      nb_sendcmd $frombot ja_netr+hostz "$hand $uidx 1 $victim $hmask"
    }
}

proc ja_netr+hostz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    set hmask [slindex $arg 4]
    if {($result == 0)} {
      putdcc $uidx "r+host successfull - added $hmask to $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "r+host failed - $victim is not a valid nickname"
      return 0
    } elseif {($result == 2)} {
      putdcc $uidx "r+host failed - $hmask couldn't be added to $victim"
      return 0
    }
}

# r+user v1.0
proc ja_dccr+user {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set hmask [lindex $arg 1]
    if {($victim == "")} {
      putdcc $idx "Usage: .+user <user> <hostmask>"
      return 0    
    }
    putcmdlog "#$hand# r+user $victim $hmask"
    nb_sendcmd $nb_control ja_netr+user "$hand $idx $victim $hmask"
    if {[nb_control]} {
      if {($channel == "")} {
        adduser $victim
      } else {
        adduser $victim $hmask
      }
    }
}

proc ja_netr+user {frombot arg} {
    set uidx [slindex $arg 1]
    set victim [slindex $arg 2]
    set hmask [slindex $arg 3]
    if {[validuser $victim]} {
      nb_sendcmd $frombot ja_netr+userz "$hand $uidx 1 $victim"
    } else {
      if {($hmask == "")} {
        if {[adduser $victim]} {
          putlog "netbots \[[slindex $arg 0]@$frombot\]: +user $victim"
          nb_sendcmd $frombot ja_netr+userz "$hand $uidx 0 $victim"
        } else {
          nb_sendcmd $frombot ja_netr+userz "$hand $uidx 2 $victim"
        }
      } else {
        if {[adduser $victim $hmask]} {
          putlog "netbots \[[slindex $arg 0]@$frombot\]: +user $victim $hmask"
          nb_sendcmd $frombot ja_netr+userz "$hand $uidx 0 $victim $hmask"
        } else {
          nb_sendcmd $frombot ja_netr+userz "$hand $uidx 2 $victim $hmask"
        }
     }
  }
}

proc ja_netr+userz {frombot arg} {
    set uidx [slindex $arg 1]
    set result [slindex $arg 2]
    set victim [slindex $arg 3]
    set hmask [slindex $arg 3]
    if {($result == 0)} {
      if {($hmask == "")} {
        putdcc $uidx "r+user successfull - added $victim"
        return 1
      } else {
        putdcc $uidx "r+user successfull - added $victim $hmask"
        return 1
      }
    } elseif {($result == 1)} {
      putdcc $uidx "r+user failed - $victim is already in userfile"
      return 0
    } elseif {($result == 2)} {
      if {($hmask == "")} {
        putdcc $uidx "r+user failed - Couldn't add $victim to userfile"
        return 0
      } else {
        putdcc $uidx "r+user failed - Couldn't add $victim with $hmask to userfile"
        return 0       
      }
    }
}

# rbotattr v1.0
proc ja_dccrbotattr {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set flags [lindex $arg 1]
	set channel [lindex $arg 2]
    if {($victim == "") || ($flags =="")} {
      putdcc $idx "Usage: .rbotattr <bot> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
      return 0    
    }
    if {$channel == ""} {
    # no channel, we check for +/- at the begining
      if {![regexp -- {^(\+|-)} $flags]} {
        # uncool, it's not here
        putdcc $idx "Usage: .rbotattr <bot> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
        return 0
      }
    } else {
    # damn, there's a channel, lets' check for |+/|- somewhere in the string
      if {[regexp -- {(\|\+|\|-)} $flags]} {
        # found that's cool - can we finish?
        # no there might be something at the begining! if it is, it should be a +/-
        if {![regexp -- {^(\|)} $flags]} {
          # well, there is something at the begining..and it's not a dash!
          # we have to test now, if it's +/-. that would be okay, everything else not.
          if {![regexp -- {^(\+|-)} $flags]} {
            # it's not! Bastard!
            putdcc $idx "Usage: .rbotattr <bot> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
            return 0
          }
        }
      } else {
        # no |+/|- found!
        putdcc $idx "Usage: .rbotattr <bot> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
        return 0
      }
    }
  	if {$channel == ""} {
      set tmp [string range $flags 1 end]
      foreach flag [split $tmp ""] {
        if {![regexp -- {(s|p|g|h|a|l|r|i)+} $flag]} {
          putdcc $idx "$flag is an invalid flag. Please do a .help whois"
          return 0
        }
      }
    } else {
      if {[regexp -- {^(\|)} $flags]} {
        set tmp [string range $flags 2 end]
        foreach flag [split $tmp ""] {
          if {![regexp -- {s} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
      } else {
        set wts [string first "|" $flags]
        set globf [string range $flags 1 [incr wts -1]]
        foreach flag [split $globf ""] {
          if {![regexp -- {(s|p|g|h|a|l|r|i)+} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
        set chanf [string range $flags [incr wts +3] end]
        foreach flag [split $chanf ""] {
          if {![regexp -- {s} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
      }
    }
    putcmdlog "#$hand# rbotattr $arg"
    nb_sendcmd $nb_control ja_netrbotattr "$hand $idx $victim $flags $channel"
    if {[nb_control]} {
    	if {$channel == ""} {
          botattr $victim $flags
        } else {
          botattr $victim $flags $channel
        }
    }
}

proc ja_netrbotattr {frombot arg} {
  set hand [lindex $arg 0]
  set uidx [lindex $arg 1]
  set victim [lindex $arg 2]
  set flags [lindex $arg 3]
  set channel [lindex $arg 4]
  if {[validuser $victim]} {
    if {$channel == ""} {
      putlog "netbots \[[slindex $arg 0]@$frombot\]: botattr $victim $flags"
      botattr $victim $flags
      nb_sendcmd $frombot ja_netrbotattrz "$hand $uidx 0 $victim $flags $channel"
    } else {
      if {[validchan $channel]} {
        putlog "netbots \[[slindex $arg 0]@$frombot\]: botattr $victim $flags $channel"
        botattr $victim $flags $channel
        nb_sendcmd $frombot ja_netrbotattrz "$hand $uidx 1 $victim $flags $channel"
      } else {
        nb_sendcmd $frombot ja_netrbotattrz "$hand $uidx 3 $victim $flags $channel"
      }
    }
  } else {
    nb_sendcmd $frombot ja_netrbotattrz "$hand $uidx 2 $victim $flags $channel"
  }
}

proc ja_netrbotattrz {frombot arg} {
	set uidx [lindex $arg 1]
	set result [lindex $arg 2]
	set victim [lindex $arg 3]
	set flags [lindex $arg 4]
	set channel [lindex $arg 5]
   if {($result == 0)} {
      putdcc $uidx "rbotattr successfull - added $flags to $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "rbotattr successfull - added $flags to $victim on $channel"
      return 1
    } elseif {($result == 2)} {
      putdcc $uidx "rbotattr failed - $victim is not a valid bot"
      return 0
    } elseif {($result == 3)} {
      putdcc $uidx "rbotattr failed - $channel is not a valid channel"
      return 0
    }
}

# rchattr v2.0
proc ja_dccrchattr {hand idx arg} {
global nb_control
	set victim [lindex $arg 0]
	set flags [lindex $arg 1]
	set channel [lindex $arg 2]
    if {($victim == "") || ($flags =="")} {
      putdcc $idx "Usage: .rchattr <nickname> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
      return 0    
    }
    if {$channel == ""} {
    # no channel, we check for +/- at the begining
      if {![regexp -- {^(\+|-)} $flags]} {
        # uncool, it's not here
        putdcc $idx "Usage: .rchattr <nickname> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
        return 0
      }
    } else {
    # damn, there's a channel, lets' check for |+/|- somewhere in the string
      if {[regexp -- {(\|\+|\|-)} $flags]} {
        # found that's cool - can we finish?
        # no there might be something at the begining! if it is, it should be a +/-
        if {![regexp -- {^(\|)} $flags]} {
          # well, there is something at the begining..and it's not a dash!
          # we have to test now, if it's +/-. that would be okay, everything else not.
          if {![regexp -- {^(\+|-)} $flags]} {
            # it's not! Bastard!
            putdcc $idx "Usage: .rchattr <nickname> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
            return 0
          }
        }
      } else {
        # no |+/|- found!
        putdcc $idx "Usage: .rchattr <nickname> <+/-><flag(s)>\[|\]<+/-><flag(s)> \[channel\]"
        return 0
      }
    }
    if {[regexp -- {n} $flags]} {
      putdcc $idx "Owner-flag (+n) mustn't be set remotely."
      return 0
    }
  	if {$channel == ""} {
      set tmp [string range $flags 1 end]
      foreach flag [split $tmp ""] {
        if {![regexp -- {(o|m|t|x|j|p|c|u|d|k|f|v|a|h|w|g|q|[A-Z])+} $flag]} {
          putdcc $idx "$flag is an invalid flag. Please do a .help whois"
          return 0
        }
      }
    } else {
      if {[regexp -- {^(\|)} $flags]} {
        set tmp [string range $flags 2 end]
        foreach flag [split $tmp ""] {
          if {![regexp -- {(m|o|d|f|k|v|q|a|w|g|[A-Z])+} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
      } else {
        set wts [string first "|" $flags]
        set globf [string range $flags 1 [incr wts -1]]
        foreach flag [split $globf ""] {
          if {![regexp -- {(o|m|t|x|j|p|c|u|d|k|f|v|a|h|w|g|q|[A-Z])+} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
        set chanf [string range $flags [incr wts +3] end]
        foreach flag [split $chanf ""] {
          if {![regexp -- {(m|o|d|f|k|v|q|a|w|g|[A-Z])+} $flag]} {
            putdcc $idx "$flag is an invalid channel flag. Please do a .help whois"
            return 0
          }
        }
      }
    }
    putcmdlog "#$hand# rchattr $arg"
    nb_sendcmd $nb_control ja_netrchattr "$hand $idx $victim $flags $channel"
    if {[nb_control]} {
      if {$channel == ""} {
        chattr $victim $flags
      } else {
        chattr $victim $flags $channel
      }
    }
}

proc ja_netrchattr {frombot arg} {
  set hand [lindex $arg 0]
  set uidx [lindex $arg 1]
  set victim [lindex $arg 2]
  set flags [lindex $arg 3]
  set channel [lindex $arg 4]
  if {[validuser $victim]} {
    if {$channel == ""} {
      putlog "netbots \[[slindex $arg 0]@$frombot\]: chattr $victim $flags"
      botattr $victim $flags
      nb_sendcmd $frombot ja_netrchattrz "$hand $uidx 0 $victim $flags $channel"
    } else {
      if {[validchan $channel]} {
        putlog "netbots \[[slindex $arg 0]@$frombot\]: chattr $victim $flags $channel"
        botattr $victim $flags $channel
        nb_sendcmd $frombot ja_netrchattrz "$hand $uidx 1 $victim $flags $channel"
      } else {
        nb_sendcmd $frombot ja_netrchattrz "$hand $uidx 3 $victim $flags $channel"
      }
    }
  } else {
    nb_sendcmd $frombot ja_netrchattrz "$hand $uidx 2 $victim $flags $channel"
  }
}

proc ja_netrchattrz {frombot arg} {
	set uidx [lindex $arg 1]
	set result [lindex $arg 2]
	set victim [lindex $arg 3]
	set flags [lindex $arg 4]
	set channel [lindex $arg 5]
   if {($result == 0)} {
      putdcc $uidx "rchattr successfull - added $flags to $victim"
      return 1
    } elseif {($result == 1)} {
      putdcc $uidx "rchattr successfull - added $flags to $victim on $channel"
      return 1
    } elseif {($result == 2)} {
      putdcc $uidx "rchattr failed - $victim is not a valid user"
      return 0
    } elseif {($result == 3)} {
      putdcc $uidx "rchattr failed - $channel is not a valid channel"
      return 0
    }
}

# rdie v1.0
proc ja_dccrdie {hand idx arg} {
global nb_control
	set reason ""
	set sure [lindex $arg 0]
	set reason [lindex $arg 1]
	if {$sure != "really"} {
		putdcc $idx "Usage: .rdie really \[reason\]"
		return 0
	}
    putcmdlog "#$hand# rdie $arg"
    nb_sendcmd $nb_control ja_netrdie "$hand $reason"
    if {[nb_control]} {
      die $reason
    }
	return 1
}

proc ja_netrdie {frombot arg} {
	set reason [lindex $arg 1]
    putlog "netbots \[[slindex $arg 0]@$frombot\]: die $reason"
	die $reason
    return 1
}

# rrestart v1.0
proc ja_dccrrestart {hand idx arg} {
global nb_control
    putcmdlog "#$hand# rrestart"
    nb_sendcmd $nb_control ja_netrrestart "$hand"
    if {[nb_control]} {
      restart
    }
	return 1
}

proc ja_netrrestart {frombot arg} {
    putlog "netbots \[[slindex $arg 0]@$frombot\]: restart"
	restart
    return 1
}



set nb_netcmds(ja_netr-bot) "ja_netr-bot"
set nb_netcmds(ja_netr-botz) "ja_netr-botz"
set nb_netcmds(ja_netr-host) "ja_netr-host"
set nb_netcmds(ja_netr-hostz) "ja_netr-hostz"
set nb_netcmds(ja_netr-user) "ja_netr-user"
set nb_netcmds(ja_netr-userz) "ja_netr-userz"
set nb_netcmds(ja_netr+bot) "ja_netr+bot"
set nb_netcmds(ja_netr+botz) "ja_netr+botz"
set nb_netcmds(ja_netr+host) "ja_netr+host"
set nb_netcmds(ja_netr+hostz) "ja_netr+hostz"
set nb_netcmds(ja_netr+user) "ja_netr+user"
set nb_netcmds(ja_netr+userz) "ja_netr+userz"
set nb_netcmds(ja_netrbotattr) "ja_netrbotattr"
set nb_netcmds(ja_netrbotattrz) "ja_netrbotattrz"
set nb_netcmds(ja_netrchattr) "ja_netrchattr"
set nb_netcmds(ja_netrchattrz) "ja_netrchattrz"
set nb_netcmds(ja_netrdie) "ja_netrdie"
set nb_netcmds(ja_netrrestart) "ja_netrrestart"

bind dcc m checkhosts ja_checkhosts
bind dcc m checkpass ja_checkpass
bind dcc n copyflags ja_copyflags
bind dcc n mchattr ja_mchattr

if {!$ja_local} {
    unbind dcc m checkhosts ja_checkhosts
    unbind dcc m checkpass ja_checkpass
    unbind dcc n copyflags ja_copyflags
    unbind dcc n mchattr ja_mchattr
}

bind dcc n r+user ja_dccr+user
bind dcc n r-user ja_dccr-user
bind dcc n r+bot ja_dccr+bot
bind dcc n r-bot ja_dccr-bot
bind dcc n r+host ja_dccr+host
bind dcc n r-host ja_dccr-host
bind dcc n rbotattr ja_dccrbotattr
bind dcc n rchattr ja_dccrchattr
bind dcc n rdie ja_dccrdie
bind dcc n rrestart ja_dccrrestart

if {!$ja_remote} {
    unbind dcc n r-user ja_dccr-user
    unbind dcc n r-bot ja_dccr-bot
    unbind dcc n r-host ja_dccr-host
    unbind dcc n r+bot ja_dccr+bot
    unbind dcc n r+user ja_dccr+user
    unbind dcc n r+host ja_dccr+host
    unbind dcc n rbotattr ja_dccrbotattr
    unbind dcc n rchattr ja_dccrchattr
    unbind dcc n rdie ja_dccrdie
    unbind dcc n rrestart ja_dccrrestart
}